#==============================================================================
# ★ RGSS3-Extension
# LNX11bskin_リフォーム・バトルステータス#スキン設定
# 　☆「スタンダード」プリセット
#
# 　このスクリプトは LNX11b_リフォーム・バトルステータス の設定スクリプトです。
# 　LNX11b の動作に必須ですので、必ず導入してください。
# 　スキンファイルは Graphics/System にインポートしてください。
#   
# 　version   : 1.00 (12/02/28)
# 　author    : ももまる
# 　reference : http://peachround.blog.fc2.com/blog-entry-12.html
#
#==============================================================================

module LNX11
  #--------------------------------------------------------------------------
  # ● 基本設定
  #--------------------------------------------------------------------------
  # ステータス全体の座標調整  :x = X座標 / :y = Y座標
  STATUS_OFFSET = {:x => 60, :y => 0}
  
  # アクター1人あたりのステータス幅
  STATUS_WIDTH  = 64
end
module LNX11b
  #--------------------------------------------------------------------------
  # ● メインスキン
  #--------------------------------------------------------------------------
  MAIN_SKIN = {
  :enabled => true,                # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_mainskin_fr",   # スキンのファイル名
  :x_fixed => false,   # X 座標を自動調整しない   true = 有効 / false = 無効
  :x =>  2, :y => 22, :z =>  0,    # XYZ 座標
  }
  #--------------------------------------------------------------------------
  # ● ステートスキン
  #--------------------------------------------------------------------------
  STATE_SKIN = {
  :enabled => true,                # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_stateskin",  # スキンのファイル名
  :x_fixed => true,      # X 座標を自動調整しない true = 有効 / false = 無効
  :x => -80, :y => -18, :z => 2,   # XYZ 座標
  }
  #--------------------------------------------------------------------------
  # ● ステートアイコン
  #--------------------------------------------------------------------------
  STATE_ICON = {
  # ○ 基本
  :enabled => true,    # 表示するか？           true = 表示 / false = 非表示
  :x_fixed => true,    # X 座標を自動調整しない true = 有効 / false = 無効
  :x => -76, :y => -8, :z => 1,    # XYZ 座標
  # ○ 機能
  :size             => 4,          # 一度に表示するアイコン数
  :scroll_interval  => 90,         # スクロール速度(フレーム)
  :background_color => Color.new(0, 0, 0, 0),  # ステートアイコンの背景色
  }
  #--------------------------------------------------------------------------
  # ● 名前 
  #--------------------------------------------------------------------------
  NAME = {
  # ○ 基本
  :enabled => false,  # 表示するか？           true = 表示 / false = 非表示
  :x_fixed => true,   # X 座標を自動調整しない true = 有効 / false = 無効
  :x => -76, :y => -8, :z => 2,     # XYZ 座標
  # ○ 機能
  :bitmap_name => false, # 名前表示に画像を使用する true = 有効 / false = 無効
  
    # bitmap_name => false :  名前表示に画像を使用しない場合
    :width  => 96, :height => 20, # 名前のサイズ :width = 幅 / :height = 高さ
    :name   => ["VL Gothic"], # フォント名
    :size   => 16,            # フォントサイズ
    :bold   => false,         # 太字    true = 有効 / false = 無効
    :italic => false,         # 斜体    true = 有効 / false = 無効
    :align  => 0,             # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
    # 描画色([文字色, 縁取り色])
    :color  => [Color.new(255, 255, 255), Color.new(0, 0, 0, 192)],
    
  # ステートが表示されている時、自動的に隠す true = 有効 / false = 無効
  :auto_hide   => true
  }
  #--------------------------------------------------------------------------
  # ● HP ゲージ
  #--------------------------------------------------------------------------
  HP_GAUGE_FORMAT = {
  # ○ 基本
  :enabled => true,              # 表示するか？   true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_hpgauge",  # ゲージのファイル名
  :x_fixed => false,     # X 座標を自動調整しない true = 有効 / false = 無効
  :x => 6, :y => 33, :z => 2,    # XYZ 座標
  # ○ 機能
  :number_link => true,  # ゲージ量を表示されている表示と一致させる
  :speed　     => 4.0,   # ゲージ変化スピード 1.0以上(一致させない場合)
  :type        => 0,     # ゲージタイプ 0 = 横ゲージ / 1 = セルゲージ
  
    # type => 0 : 横ゲージの設定
    :crisis    => 0.25,  # ピンチ状態とみなすパーセンテージ
    :s_speed   => 0.2,   # サブゲージ変化スピード(pixel per frame)
    
    # type => 1 : セルゲージの設定
    :width => 36, :height => 36, # セルのサイズ :width = 幅 / :height = 高さ
    :pattern   => :auto, # パターン数 :auto を指定するとパターン数を自動判定
    :direction => 1,     # パターンの方向 0 = 横 / 1 = 縦
  }
  #--------------------------------------------------------------------------
  # ● MP ゲージ
  #--------------------------------------------------------------------------
  MP_GAUGE_FORMAT = {
  # ○ 基本
  :enabled => true,             # 表示するか？     true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_mpgauge", # ゲージのファイル名
  :x_fixed => false,      # X 座標を自動調整しない true = 有効 / false = 無効
  :x => 6, :y => 55, :z => 2,   # XYZ 座標
  # ○ 機能
  :number_link => true,   # ゲージ量を表示されている表示と一致させる
  :speed　     => 4.0,    # ゲージ変化スピード 1.0以上(一致させない場合)  
  :type        => 0,      # ゲージタイプ 0 = 横ゲージ / 1 = セルゲージ
  
    # type => 0 : 横ゲージの設定
    :crisis    => 0.25,   # ピンチ状態とみなすパーセンテージ
    :s_speed   => 0.2,    # サブゲージ変化スピード(pixel per frame)  
    
    # type => 1 : セルゲージの設定
    :width     => 36, :height => 36, # セルのサイズ :width = 幅 / :height = 高さ
    :pattern   => :auto,  # パターン数 :auto を指定するとパターン数を自動判定
    :direction => 1,      # パターンの方向 0 = 横 / 1 = 縦
  }
  #--------------------------------------------------------------------------
  # ● TP ゲージ
  #--------------------------------------------------------------------------
  TP_GAUGE_FORMAT = {
  # ○ 基本
  :enabled => true,              # 表示するか？   true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_tpgauge", # ゲージのファイル名
  :x_fixed => false,     # X 座標を自動調整しない true = 有効 / false = 無効
  :x => 6, :y => 77, :z => 3,    # XYZ 座標
  # ○ 機能
  :number_link => true,  # ゲージ量を表示されている表示と一致させる
  :speed　     => 4.0,   # ゲージ変化スピード 1.0以上(一致させない場合)  
  :type        => 0,     # ゲージタイプ 0 = 横ゲージ / 1 = セルゲージ
  
    # type => 0 : 横ゲージの設定
    :crisis    => 0.25,  # ピンチ状態とみなすパーセンテージ
    :s_speed   => 0.2,   # サブゲージ変化スピード(pixel per frame)
    
    # type => 1 : セルゲージの設定
    :width     => 36, :height => 36, # セルのサイズ :width = 幅 / :height = 高さ
    :pattern   => :auto, # パターン数 :auto を指定するとパターン数を自動判定
    :direction => 1,     # パターンの方向 0 = 横 / 1 = 縦
  }
  #--------------------------------------------------------------------------
  # ● HP 数字
  #--------------------------------------------------------------------------
  HP_NUMBER_FORMAT = {
  # ○ 基本
  :enabled => true,                 # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_largenumber", # 数字のファイル名
  :x_fixed => false,   # X 座標を自動調整しない    true = 有効 / false = 無効
  :x => 64, :y => 17, :z => 4,      # XYZ 座標
  # ○ 機能
  :digit   => 4,       # 桁数
  :spacing => -2,      # 字間
  :align   => 2,       # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  :speed   => 12,      # 数字の変化スピード 1 以上
  :crisis  => 0.25,    # ピンチ状態とみなすパーセンテージ
  :dead    => 0.0,     # 死亡状態とみなすパーセンテージ
  }
  #--------------------------------------------------------------------------
  # ● MP 数字
  #--------------------------------------------------------------------------
  MP_NUMBER_FORMAT = {
  # ○ 基本
  :enabled => true,                 # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_largenumber", # 数字のファイル名
  :x_fixed => false,   # X 座標を自動調整しない    true = 有効 / false = 無効
  :x => 64, :y => 39, :z => 4,      # XYZ 座標
  # ○ 機能
  :digit   => 4,       # 桁数
  :spacing => -2,      # 字間
  :align   => 2,       # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  :speed   => 12,      # 数字の変化スピード 1 以上
  :crisis  => 0.25,    # ピンチ状態とみなすパーセンテージ
  :dead    => -1.0,    # 死亡状態とみなすパーセンテージ
  }
  #--------------------------------------------------------------------------
  # ● TP 数字
  #--------------------------------------------------------------------------
  TP_NUMBER_FORMAT = {
  # ○ 基本
  :enabled => true,                 # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_largenumber", # 数字のファイル名
  :x_fixed => false,   # X 座標を自動調整しない    true = 有効 / false = 無効
  :x => 64, :y => 61, :z => 5,      # XYZ 座標
  # ○ 機能
  :digit   => 4,       # 桁数
  :spacing => -2,      # 字間
  :align   => 2,       # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  :speed   => 12,      # 数字の変化スピード 1 以上
  :crisis  => -1.0,    # ピンチ状態とみなすパーセンテージ
  :dead    => -1.0,    # 死亡状態とみなすパーセンテージ
  }
  #--------------------------------------------------------------------------
  # ● 最大 HP 数字
  #--------------------------------------------------------------------------
  MAXHP_NUMBER_FORMAT = {
  # ○ 基本
  :enabled => false,                # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_smallnumber", # 数字のファイル名
  :x_fixed => false,   # X 座標を自動調整しない    true = 有効 / false = 無効
  :x => 64, :y => 23, :z => 6,      # XYZ 座標
  # ○ 機能
  :digit   => 4,       # 桁数
  :spacing => -2,      # 字間
  :align   => 0,       # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  :speed   => 12,      # 数字の変化スピード 1 以上
  :crisis  => 0.25,    # ピンチ状態とみなすパーセンテージ
  :dead    => 0.0,     # 死亡状態とみなすパーセンテージ
  }  
  #--------------------------------------------------------------------------
  # ● 最大 MP 数字
  #--------------------------------------------------------------------------
  MAXMP_NUMBER_FORMAT = {
  # ○ 基本
  :enabled => false,                # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_smallnumber", # 数字のファイル名
  :x_fixed => false,   # X 座標を自動調整しない    true = 有効 / false = 無効
  :x => 64, :y => 45, :z => 6,      # XYZ 座標
  # ○ 機能
  :digit   => 4,       # 桁数
  :spacing => -2,      # 字間
  :align   => 0,       # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  :speed   => 12,      # 数字の変化スピード 1 以上
  :crisis  => 0.25,    # ピンチ状態とみなすパーセンテージ
  :dead    => -1.0,    # 死亡状態とみなすパーセンテージ
  }  
  #--------------------------------------------------------------------------
  # ● 最大 TP 数字
  #--------------------------------------------------------------------------
  MAXTP_NUMBER_FORMAT = {
  # ○ 基本
  :enabled => false,                # 表示するか？ true = 表示 / false = 非表示
  :bitmap  => "Lnx11b_smallnumber", # 数字のファイル名
  :x_fixed => false,   # X 座標を自動調整しない    true = 有効 / false = 無効
  :x => 64, :y => 61, :z => 6,      # XYZ 座標
  # ○ 機能
  :digit   => 4,       # 桁数
  :spacing => -2,      # 字間
  :align   => 0,       # 文字揃え 0 = 左揃え / 1 = 中央揃え / 2 = 右揃え
  :speed   => 12,      # 数字の変化スピード 1 以上
  :crisis  => -1.0,    # ピンチ状態とみなすパーセンテージ
  :dead    => -1.0,    # 死亡状態とみなすパーセンテージ
  }
  #
  # 設定ここまで お疲れ様でした。
  #
end
#==============================================================================
# ■ LNXスクリプト導入情報
#==============================================================================
$lnx_include = {} if $lnx_include == nil
$lnx_include[:lnx11bskin] = 100 # version
p "OK:LNX11bskin_リフォーム・バトルステータス#スキン設定"

